@@ -14,6 +14,12 @@ G_BEGIN_DECLS
 typedef struct _PatchListClass PatchListClass;
 typedef struct _PatchList PatchList;
 
+enum
+{
+    PATCH_LIST_INDEX,		/* target is a display index */
+    PATCH_LIST_PATCH,		/* target is a patch id */
+};
+
 struct _PatchList
 {
     GtkHBox parent;
@@ -37,11 +43,16 @@ struct _PatchListClass
 GType patch_list_get_type(void);
 GtkWidget* patch_list_new(void);
 
-/* refresh the list of patches */
-void patch_list_update(PatchList* self, int index);
+/* refresh the list of available pactches; target specifies either the
+ index or patch id of the item to select, depending on the value of
+ type */
+void patch_list_update(PatchList* self, int target, int type);
 
 /* returns which patch we are currently looking at */
 int patch_list_get_current_patch(PatchList* self);
+
+/* returns the index of the patch we are currently looking at */
+int patch_list_get_current_index(PatchList* self);
     
 G_END_DECLS
 
