@@ -22,15 +22,12 @@ struct _PatchSection
     int patch;
     guint refresh;
     GtkWidget* title;
-    GtkWidget* play_button;
-    GtkWidget* panic_button;
     GtkWidget* volume_fan;
     GtkWidget* pan_fan;
     GtkWidget* pitch_fan;
     GtkWidget* range_sb;
     GtkWidget* notebook;
     GtkWidget* sample_tab;
-    GtkWidget* midi_tab;
     GtkWidget* voice_tab;
     GtkWidget* filter_tab;
     GtkWidget* vel_tab;
