@@ -35,6 +35,7 @@ static float calc_bpm (snd_seq_t* handle, int q)
      return bpm;
 }
 
+
 static void map_control(unsigned char chan, int param, float value)
 {
     static struct
@@ -55,11 +56,12 @@ static void map_control(unsigned char chan, int param, float value)
     
     int i;
      
-    for(i=0; i<sizeof(map)/sizeof(map[0]); ++i)
-	if( map[i].cc == param )
+    for (i=0; i<sizeof(map)/sizeof(map[0]); ++i)
+	if (map[i].cc == param)
 	    mixer_control(chan, map[i].param, value * map[i].scale + map[i].bias);
 }
 
+
 static void action (snd_seq_t* handle)
 {
      snd_seq_event_t* ev;
@@ -70,12 +72,12 @@ static void action (snd_seq_t* handle)
           switch (ev->type)
           {
           case SND_SEQ_EVENT_NOTEON:
-               if (ev->data.note.velocity == 0x00)
+               if (ev->data.note.velocity == 0)
                     mixer_note_off (ev->data.note.channel,
                                     ev->data.note.note);
                else
                     mixer_note_on (ev->data.note.channel,
-                                   ev->data.note.note, ev->data.note.velocity);
+                                   ev->data.note.note, ev->data.note.velocity / 127.0);
 	       break;
           case SND_SEQ_EVENT_NOTEOFF:
                mixer_note_off (ev->data.note.channel, ev->data.note.note);
@@ -103,6 +105,7 @@ static void action (snd_seq_t* handle)
      while (snd_seq_event_input_pending (handle, 0) > 0);
 }
 
+
 static void* poll_events (void* arg)
 {
      snd_seq_t* handle = arg;
@@ -125,6 +128,7 @@ static void* poll_events (void* arg)
      return 0;
 }
 
+
 static int open_seq (snd_seq_t** handle)
 {
      int portid;
@@ -153,6 +157,7 @@ static int open_seq (snd_seq_t** handle)
      return 0;
 }
 
+
 int midi_start ( )
 {
      int err;
@@ -174,6 +179,7 @@ int midi_start ( )
      return 0;
 }
 
+
 void midi_stop ( )
 {
      if (!running)
