@@ -11,6 +11,7 @@
 #include "adsr.h"
 #include "lfo.h"
 #include "driver.h"		/* for DRIVER_DEFAULT_SAMPLERATE */
+#include "midi.h"		/* for MIDI_CHANS */
 
 /* the minimum envelope release value we'll allow (to prevent clicks) */
 const float PATCH_MIN_RELEASE = 0.05;
@@ -41,7 +42,6 @@ typedef struct _PatchVoice
 {
     gboolean active;		/* whether this voices is playing or not */
     Tick     ticks;		/* at what time this voice was activated */
-    int      offset;		/* how many ticks should pass before this voice is used */
     int      relset;		/* how many ticks should pass before we release this voice (negative if N/A) */
     gboolean released;		/* whether or not we've been released or not */
     float    declick_vol;	/* current volume being applied to declick phase */
@@ -56,11 +56,6 @@ typedef struct _PatchVoice
     guint32  stepf;		/* fractional step amount */
     float    vel;		/* velocity; volume of this voice */
 
-    /* legato stuff */
-    int      retrig_ticks;	/* how many ticks until we retrigger */
-    int      retrig_note;	/* the note we will retrigger at */
-    float    retrig_vel;	/* the velocity we will retrigger at */
-
     ADSR     vol;		/* volume envelope */
     ADSR     pan;		/* panning envelope */
     ADSR     ffreq;       	/* filter frequency envelope */
@@ -77,8 +72,8 @@ typedef struct _PatchVoice
     float    flr;		/* lowpass filter buffer, right */
     float    fbl;		/* bandpass filter buffer, left */
     float    fbr;		/* bandpass filter buffer, right */
-}
-    PatchVoice;
+} PatchVoice;
+
 
 /* type for modulatable parameters */
 typedef struct _PatchParam
@@ -97,8 +92,8 @@ typedef struct _PatchParam
     float*     lfo_tab;		/* used for storing global_lfo values */
      
     float      vel_amt;		/* amount of velocity to use [0.0, 1.0] */
-}
-    PatchParam;
+} PatchParam;
+
 
 /* type for array of instruments (called patches) */
 typedef struct _Patch
@@ -146,8 +141,8 @@ typedef struct _Patch
     /* used exclusively by patch_lock functions to ensure that
      * patch_render ignores this patch */
     pthread_mutex_t mutex;
-}
-    Patch;
+} Patch;
+
 
 /* array of all patches  */
 static Patch patches[PATCH_COUNT];
@@ -226,15 +221,15 @@ inline static void patch_trigger_global_lfos ( )
 {
     int i;
 
-    debug ("retriggering...\n")
-	for (i = 0; i < PATCH_COUNT; i++)
-	{
-	    lfo_trigger (&patches[i].vol.global_lfo, &patches[i].vol.lfo_params);
-	    lfo_trigger (&patches[i].pan.global_lfo, &patches[i].pan.lfo_params);
-	    lfo_trigger (&patches[i].ffreq.global_lfo, &patches[i].ffreq.lfo_params);
-	    lfo_trigger (&patches[i].freso.global_lfo, &patches[i].freso.lfo_params);
-	    lfo_trigger (&patches[i].pitch.global_lfo, &patches[i].pitch.lfo_params);
-	}
+    debug ("retriggering...\n");
+    for (i = 0; i < PATCH_COUNT; i++)
+    {
+	lfo_trigger (&patches[i].vol.global_lfo, &patches[i].vol.lfo_params);
+	lfo_trigger (&patches[i].pan.global_lfo, &patches[i].pan.lfo_params);
+	lfo_trigger (&patches[i].ffreq.global_lfo, &patches[i].ffreq.lfo_params);
+	lfo_trigger (&patches[i].freso.global_lfo, &patches[i].freso.lfo_params);
+	lfo_trigger (&patches[i].pitch.global_lfo, &patches[i].pitch.lfo_params);
+    }
     debug ("done\n");
 }     
 /*****************************************************************************/
@@ -316,7 +311,7 @@ int patch_create (const char *name)
     deflfo.attack = 0.0;
 
     /* volume */
-    patches[id].vol.val = 1.0;
+    patches[id].vol.val = DEFAULT_VOLUME;
     patches[id].vol.env = defadsr;
     patches[id].vol.env_amt = 0.0; /* this parameter is actually ignored for volume */
     patches[id].vol.env_on = FALSE;
@@ -377,7 +372,6 @@ int patch_create (const char *name)
 
     /* default voice */
     defvoice.active = FALSE;
-    defvoice.offset = 0;
     defvoice.note = 0;
     defvoice.posi = 0;
     defvoice.posf = 0;
@@ -467,11 +461,11 @@ void patch_destroy_all ( )
     return;
 }
 
-/* place all patch ids, sorted in ascending order by display index,
-   into array 'id' and return number of patches */
+/* place all patch ids, sorted in ascending order by channels and then
+   notes, into array 'id' and return number of patches */
 int patch_dump (int **dump)
 {
-    int i, j, id, count, tmp;
+    int i, j, k, id, count, tmp;
 
     *dump = NULL;
 
@@ -491,13 +485,13 @@ int patch_dump (int **dump)
 	if (patches[id].active)
 	    (*dump)[i++] = id;
 
-    /* sort dump array by display_index in ascending order */
+    /* sort dump array by channel in ascending order */
     for (i = 0; i < count; i++)
     {
 	for (j = i; j < count; j++)
 	{
-	    if (patches[(*dump)[j]].display_index <
-		patches[(*dump)[i]].display_index)
+	    if (patches[(*dump)[j]].channel <
+		patches[(*dump)[i]].channel)
 	    {
 		tmp = (*dump)[i];
 		(*dump)[i] = (*dump)[j];
@@ -506,6 +500,31 @@ int patch_dump (int **dump)
 	}
     }
 
+    /* sort dump array by note in ascending order while preserving
+     * existing channel order */
+    for (i = 0; i < MIDI_CHANS; i++)
+    {
+	for (j = 0; j < count; j++)
+	{
+	    if (patches[(*dump)[j]].channel != i)
+		continue;
+	    
+	    for (k = j; k < count; k++)
+	    {
+		if (patches[(*dump)[k]].channel != i)
+		    continue;
+
+		if (patches[(*dump)[k]].note <
+		    patches[(*dump)[j]].note)
+		{
+		    tmp = (*dump)[j];
+		    (*dump)[j] = (*dump)[k];
+		    (*dump)[k] = tmp;
+		}
+	    }
+	}
+    }
+
     return count;
 }
 
@@ -807,7 +826,7 @@ void patch_sync (float bpm)
 
 /* a helper function to release all voices matching a given criteria
  * (if note is a negative value, all active voices will be released) */
-inline static void patch_release_patch (Patch* p, int note, int offset)
+inline static void patch_release_patch (Patch* p, int note)
 {
     int i;
     PatchVoice* v;
@@ -820,7 +839,7 @@ inline static void patch_release_patch (Patch* p, int note, int offset)
 
 	    /* we don't really release here, that's the job of
 	     * advance( ); we just tell it *when* to release */
-	    v->relset = offset;
+	    v->relset = 0;
 
 	    if (p->mono && p->legato)
 		v->relset += legato_lag;
@@ -830,7 +849,7 @@ inline static void patch_release_patch (Patch* p, int note, int offset)
 
 /* a helper function to cut all patches whose cut_by value matches the
  * cut value for the given patch */
-inline static void patch_cut_patch (Patch* p, int offset)
+inline static void patch_cut_patch (Patch* p)
 {
     int i;
 
@@ -843,7 +862,7 @@ inline static void patch_cut_patch (Patch* p, int offset)
     {
 	if (patches[i].active && patches[i].cut_by == p->cut)
 	{
-	    patch_release_patch (&patches[i], -69, offset);
+	    patch_release_patch (&patches[i], -69);
 	}
     }
 }
@@ -851,12 +870,18 @@ inline static void patch_cut_patch (Patch* p, int offset)
 /* a helper function to prepare a voice's pitch information */
 inline static void prepare_pitch(Patch* p, PatchVoice* v, int note)
 {
+    double scale;		/* base pitch scaling factor */
+
+    /* this applies the tuning factor */
+    scale = pow (2, (p->pitch.val * p->pitch_steps) / 12.0);
+
     if (p->porta && (p->porta_secs > 0.0)
 	&& (p->last_note != note))
     {
 	/* we calculate the pitch here because we can't be certain
 	 * what the current value of the last voice's pitch is */
 	v->pitch = pow (2, (p->last_note - p->note) / 12.0);
+	v->pitch *= scale;
 	  
 	v->porta_ticks = ticks_secs_to_ticks (p->porta_secs);
 
@@ -864,13 +889,14 @@ inline static void prepare_pitch(Patch* p, PatchVoice* v, int note)
 	 * subtracting the target pitch from the initial pitch and
 	 * dividing by porta_ticks */
 	v->pitch_step = ((pow (2, (note - p->note)
-			       / 12.0))
+			       / 12.0) * scale)
 			 - v->pitch)
 	    / v->porta_ticks;
     }
     else
     {
 	v->pitch = pow (2, (note - p->note) / 12.0);
+	v->pitch *= scale;
 	v->porta_ticks = 0;
     }
 
@@ -880,14 +906,13 @@ inline static void prepare_pitch(Patch* p, PatchVoice* v, int note)
      * against ourself) */
     p->last_note = note;
 
-    v->pitch *= pow (2, (p->pitch.val * p->pitch_steps) / 12.0); /* scale to base pitch */
+    /* give our step variables their initial values */
     v->stepi = v->pitch;
     v->stepf = (v->pitch - v->stepi) * (0xFFFFFFFFU); /* maximum value of guint32 */
 }
 
 /* a helper function to activate a voice for a patch */
-inline static void patch_trigger_patch (Patch* p, int note, float vel,
-					Tick ticks, int offset)
+inline static void patch_trigger_patch (Patch* p, int note, float vel, Tick ticks)
 {
     int i;
     PatchVoice* v;
@@ -913,6 +938,11 @@ inline static void patch_trigger_patch (Patch* p, int note, float vel,
 
 	    if (!p->voices[i].active)
 		empty = i;
+
+	    /* we need to make sure that we don't grab a voice that is
+	     * busy trying to release itself, otherwise we'll take it
+	     * over, only to be promptly reduced to zero volume and
+	     * shut down */
 	    else if (!p->voices[i].released)
 		break;
 	}
@@ -926,16 +956,22 @@ inline static void patch_trigger_patch (Patch* p, int note, float vel,
 		index = gzr;
 	    else
 		index = empty;
+
+	    index = 0;
 	}
 	else
 	{
-	    p->voices[i].ticks = ticks;
-	    p->voices[i].retrig_ticks = offset;
-	    p->voices[i].retrig_note = note;
-	    p->voices[i].retrig_vel = vel;
-	    p->voices[i].relset = -1; /* cancel any pending release */
+	    v = &p->voices[i];
+	    
+	    v->ticks = ticks;
+	    v->note = note;
+	    v->vel = vel;
+	    v->relset = -1;	/* cancel any pending release */
 
-	    return;
+	    prepare_pitch(p, v, note);
+
+	    return;		/* the rest of this function is
+				 * irrelevant now, fuck it */
 	}
     }
     else
@@ -971,11 +1007,10 @@ inline static void patch_trigger_patch (Patch* p, int note, float vel,
     
     /* shutdown any running voices if monophonic */
     if (p->mono)
-	patch_release_patch(p, -1, offset);
+	patch_release_patch(p, -1);
 
     /* fill in our voice */
     v->ticks = ticks;
-    v->offset = offset;
     v->relset = -1;		/* N/A at this time */
     v->released = FALSE;
     v->declick_vol = 1.0;
@@ -984,8 +1019,6 @@ inline static void patch_trigger_patch (Patch* p, int note, float vel,
     v->fbl = 0;
     v->flr = 0;
     v->fbr = 0;
-    v->retrig_ticks = -1;	/* N/A for non-legato patches */
-    v->retrig_note = -1;	/* N/A for non-legato patches */
     v->vel = vel;
 
     /* setup direction */
@@ -1108,7 +1141,7 @@ inline static void pan (Patch * p, PatchVoice * v, int index, float *l, float *r
 inline static void filter (Patch* p, PatchVoice* v, int index,  float* l,
 			   float* r)
 {
-    float ffreq, freso;
+    float ffreq, freso, logreso;
 
     /* get filter cutoff frequency */
     ffreq = p->ffreq.val;
@@ -1173,13 +1206,16 @@ inline static void filter (Patch* p, PatchVoice* v, int index,  float* l,
     else if (freso < 0.0)
 	freso = 0.0;
 
+    /* logify */
+    logreso = log_volume(freso);
+
     /* left */
-    v->fbl = freso * v->fbl + ffreq * (*l - v->fll);
+    v->fbl = logreso * v->fbl + ffreq * (*l - v->fll);
     v->fll += ffreq * v->fbl;
     *l = v->fll;
 
     /* right */
-    v->fbr = freso * v->fbr + ffreq * (*r - v->flr);
+    v->fbr = logreso * v->fbr + ffreq * (*r - v->flr);
     v->flr += ffreq * v->fbr;
     *r = v->flr;
 }
@@ -1187,7 +1223,8 @@ inline static void filter (Patch* p, PatchVoice* v, int index,  float* l,
 /* a helper routine to adjust the volume of a frame */
 inline static int gain (Patch* p, PatchVoice* v, int index, float* l, float* r)
 {
-    float vol;
+    float vol = 0.0;
+    float logvol = 0.0;
 
     /* first, we use our set value as a base */
     vol = p->vol.val;
@@ -1221,7 +1258,6 @@ inline static int gain (Patch* p, PatchVoice* v, int index, float* l, float* r)
     /* velocity should be the last parameter considered because it
      * has the most "importance" */
     vol = lerp (vol, vol * v->vel, p->vol.vel_amt);
-    
 
     /* clip */
     if (vol > 1.0)
@@ -1229,10 +1265,12 @@ inline static int gain (Patch* p, PatchVoice* v, int index, float* l, float* r)
     else if (vol < 0.0)
 	vol = 0.0;
 
-    /* adjust volume */
-    *l *= vol;
-    *r *= vol;
+    /* as a last step, make logarithmic */
+    logvol = log_volume(vol);
 
+    /* adjust volume */
+    *l *= logvol;
+    *r *= logvol;
 
     /* check to see if we've finished a release */
     if (v->released && vol < ALMOST_ZERO)
@@ -1262,7 +1300,7 @@ inline static int advance (Patch* p, PatchVoice* v, int index)
     /* base pitch value */
     pitch = v->pitch;
 
-    if( p->pitch_bend )
+    if (p->pitch_bend)
     {
 	recalc = TRUE;
 	pitch *= p->pitch_bend;
@@ -1400,19 +1438,6 @@ inline static int advance (Patch* p, PatchVoice* v, int index)
 	}
     }
 
-    /* otherwise, check to see if we should retrigger */
-    else if (p->mono && p->legato && v->retrig_ticks >= 0)
-    {
-	if (v->retrig_ticks == 0)
-	{
-	    prepare_pitch(p, v, v->retrig_note);
-	    v->note = v->retrig_note;
-	    v->vel = v->retrig_vel;
-	}
-
-	--v->retrig_ticks;
-    }
-
     return 0;
 }
 
@@ -1468,7 +1493,7 @@ inline static void patch_render_patch (Patch* p, float* buf, int nframes)
 	}
 
 	done = FALSE;
-	for (j = v->offset; j < nframes && done == FALSE; j++)
+	for (j = 0; j < nframes && done == FALSE; j++)
 	{
 	    /* process samples */
 	    pitchscale (p, v,    &l, &r);
@@ -1493,19 +1518,6 @@ inline static void patch_render_patch (Patch* p, float* buf, int nframes)
 	if (done)
 	    v->active = FALSE;
 
-	/* decrease the voices offset if applicable */
-	if (v->offset > 0)
-	{
-	    if (v->offset > nframes)
-	    {
-		v->offset -= nframes;
-	    }
-	    else
-	    {
-		v->offset = 0;
-	    }
-	}
-
 	/* overflows bad, OVERFLOWS BAD! */
 	if (v->active && (v->posi < 0 || v->posi >= p->sample->frames))
 	{
@@ -1517,7 +1529,7 @@ inline static void patch_render_patch (Patch* p, float* buf, int nframes)
 }
 
 /* deactivate all active patches matching given criteria */
-void patch_release (int chan, int note, int offset)
+void patch_release (int chan, int note)
 {
     int i;
 
@@ -1532,7 +1544,7 @@ void patch_release (int chan, int note, int offset)
 		    && note <= patches[i].upper_note)))
 	{
 
-	    patch_release_patch (&patches[i], note, offset);
+	    patch_release_patch (&patches[i], note);
 	}
     }
 
@@ -1540,7 +1552,7 @@ void patch_release (int chan, int note, int offset)
 }
 
 /* deactivate a single patch with a given id */
-void patch_release_with_id (int id, int note, int offset)
+void patch_release_with_id (int id, int note)
 {
     if (id < 0 || id >= PATCH_COUNT)
 	return;
@@ -1549,7 +1561,7 @@ void patch_release_with_id (int id, int note, int offset)
     if (patches[id].play_mode & PATCH_PLAY_SINGLESHOT)
 	return;
 
-    patch_release_patch (&patches[id], note, offset);
+    patch_release_patch (&patches[id], note);
     return;
 }
 
@@ -1579,8 +1591,7 @@ void patch_render (float *buf, int nframes)
 }
 
 /* triggers all patches matching criteria */
-void patch_trigger (int chan, int note, float vel, Tick ticks,
-		    int offset)
+void patch_trigger (int chan, int note, float vel, Tick ticks)
 {
     static int idp[PATCH_COUNT];	/* holds all patches to be activated */
     int i, j;
@@ -1609,24 +1620,27 @@ void patch_trigger (int chan, int note, float vel, Tick ticks,
 
     /* do cuts */
     for (i = 0; i < j; i++)
-	patch_cut_patch (&patches[idp[i]], offset);
+	patch_cut_patch (&patches[idp[i]]);
 
     /* do triggers */
     for (i = 0; i < j; i++)
-	patch_trigger_patch (&patches[idp[i]], note, vel, ticks, offset);
+	patch_trigger_patch (&patches[idp[i]], note, vel, ticks);
 }
 
 /* activate a single patch with given id */
-void patch_trigger_with_id (int id, int note, float vel, Tick ticks,
-			    int offset)
+void patch_trigger_with_id (int id, int note, float vel, Tick ticks)
 {
     if (id < 0 || id >= PATCH_COUNT)
 	return;
     if (!patches[id].active)
 	return;
+    if (note < patches[id].lower_note || note > patches[id].upper_note)
+	return;
+    if (!patches[id].range && note != patches[id].note)
+	return;
 
-    patch_cut_patch (&patches[id], offset);
-    patch_trigger_patch (&patches[id], note, vel, ticks, offset);
+    patch_cut_patch (&patches[id]);
+    patch_trigger_patch (&patches[id], note, vel, ticks);
     return;
 }
 
