@@ -85,13 +85,11 @@ void        patch_sync            (float bpm);
 int         patch_verify          (int id);
 
 /* playback and rendering functions  */
-void patch_release         (int chan, int note, int offset);
-void patch_release_with_id (int id, int note, int offset);
+void patch_release         (int chan, int note);
+void patch_release_with_id (int id, int note);
 void patch_render          (float* buf, int nframes);
-void patch_trigger         (int chan, int note, float vel, Tick ticks,
-			    int offset);
-void patch_trigger_with_id (int id, int note, float vel, Tick ticks,
-			    int offset);
+void patch_trigger         (int chan, int note, float vel, Tick ticks);
+void patch_trigger_with_id (int id, int note, float vel, Tick ticks);
 void patch_control	   (int chan, ControlParamType param, float value);
 
 /* envelope setters */
